# -*- coding: utf-8 -*-
"""
    blohg.utils
    ~~~~~~~~~~~

    Module with general purpose utilities for blohg.

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

import codecs
import os
import shutil

from calendar import timegm
from flask import current_app
from mercurial import commands, error
from pkg_resources import resource_filename, resource_listdir
from time import strptime, time


def create_repo(app):
    """Function to initialize a blohg repo, with the default template files
    inside.
    """

    repo_path = app.config.get('REPO_PATH')
    template_path = resource_filename('blohg', 'repo_template')
    template_rootfiles = resource_listdir('blohg', 'repo_template')

    initialized = False
    for f in template_rootfiles + ['.hg']:
        if os.path.exists(os.path.join(repo_path, f)):
            initialized = True

    if initialized:
        raise RuntimeError('repository already initialized: %s' % repo_path)

    if not os.path.exists(repo_path):
        os.makedirs(repo_path)

    for f in template_rootfiles:
        full_path = os.path.join(template_path, f)
        if os.path.isdir(full_path):
            shutil.copytree(full_path, os.path.join(repo_path, f))
        elif os.path.isfile(full_path):
            shutil.copy2(full_path, os.path.join(repo_path, f))
        else:
            raise RuntimeError('unrecognized file: %s' % full_path)

    # create a .hgignore, to avoid people to acidentally push a build/ dir
    # with stuff built with 'blohg freeze'. creating the file here because a
    # .hgignore file in the repo may cause some weird behavior that we are not
    # aware of.
    with codecs.open(os.path.join(repo_path, '.hgignore'), 'w',
                     encoding='utf-8') as fp:
        fp.write('^build/' + os.linesep)

    try:
        commands.init(app.hgui, repo_path)
    except error, err:
        raise RuntimeError('an error was occurred: %s' % err)


def parse_date(datestring):
    """blohg used to accept datetimes formated as UNIX timestamps to override
    the datetimes provided by the Mercurial API, but UNIX timestamps are hard
    to read and guess. This function allows users to write datetimes using a
    more readable format::

        YYYY-MM-DD HH:MM:SS

    UNIX timestamps are still a valid input format.
    """
    if datestring.isdigit():
        return int(datestring)
    timetuple = strptime(datestring, '%Y-%m-%d %H:%M:%S')
    return timegm(timetuple)



# -*- coding: utf-8 -*-
"""
    blohg
    ~~~~~

    Main package.

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

from flask import Flask, render_template, request
from flask.ext.babel import Babel

# import blohg stuff
from blohg.hgapi import setup_mercurial
from blohg.version import version as __version__
from blohg.views import views


def create_app(repo_path=None, hgui=None):
    """Application factory.

    :param repo_path: the path to the mercurial repository.
    :return: the WSGI application (Flask instance).
    """

    # create the app object
    app = Flask(__name__)

    # register some sane default config values
    app.config.setdefault('AUTHOR', u'Your Name Here')
    app.config.setdefault('POSTS_PER_PAGE', 10)
    app.config.setdefault('TAGLINE', u'Your cool tagline')
    app.config.setdefault('TITLE', u'Your title')
    app.config.setdefault('TITLE_HTML', u'Your HTML title')
    app.config.setdefault('CONTENT_DIR', u'content')
    app.config.setdefault('TEMPLATES_DIR', u'templates')
    app.config.setdefault('STATIC_DIR', u'static')
    app.config.setdefault('ATTACHMENT_DIR', u'content/attachments')
    app.config.setdefault('ROBOTS_TXT', True)
    app.config.setdefault('SHOW_RST_SOURCE', True)
    app.config.setdefault('POST_EXT', u'.rst')
    app.config.setdefault('OPENGRAPH', True)
    app.config.setdefault('TIMEZONE', 'UTC')
    app.config.setdefault('RST_HEADER_LEVEL', 3)

    app.config['REPO_PATH'] = repo_path

    # init mercurial stuff
    setup_mercurial(app, hgui=hgui)

    # setup extensions
    babel = Babel(app)

    @app.context_processor
    def setup_jinja2():
        return dict(
            version=__version__,
            is_post=lambda x: x.startswith('post/'),
            current_path=request.path.strip('/'),
            active_page=request.path.strip('/').split('/')[0],
            tags=app.hg.tags,
            config=app.config,
        )

    @babel.timezoneselector
    def get_timezone():
        return app.config['TIMEZONE']

    @app.errorhandler(404)
    def page_not_found(error):
        return render_template('404.html'), 404

    app.register_blueprint(views)

    return app



# -*- coding: utf-8 -*-
"""
    blohg.rst.roles
    ~~~~~~~~~~~~~~~

    Module with the custom blohg reStructuredText roles.

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

from docutils.nodes import reference, paragraph
from flask import current_app, url_for

import posixpath
import re


__all__ = ['attachment_role', 'page_role']


def attachment_role(name, rawtext, text, lineno, inliner, options={},
                    content=[]):
    """reStructuredText role that creates a Text node with the full url for an
    attachment.
    """
    label = ""
    if '|' in text:
        text, label = text.split('|')
    full_path = posixpath.join(current_app.config['ATTACHMENT_DIR'], text)
    if full_path not in current_app.hg.revision.manifest():
        msg = inliner.reporter.error('Error in "%s" role: File not found: %s.' \
                                     % (name, full_path), line=lineno)
        prb = inliner.problematic(rawtext, rawtext, msg)
        return [prb], [msg]
    url = url_for('attachments', filename=text, _external=True)
    if not label:
        label = url
    return [reference(url, label, refuri=url)], []


# \x00 means the "<" was backslash-escaped
explicit_title_re = re.compile(r'^(.+?)\s*(?<!\x00)<(.*?)>$', re.DOTALL)


def page_role(name, rawtext, text, lineno, inliner, options={}, content={}):
    """reStructuredText role to generate a link to another page/post
    use as :page:`slug`, or :page:`title <slug>`. In the first case, the
    title used will be the one of the post itself.
    """
    title = None
    target = text
    match = explicit_title_re.match(text)
    if match:
        title, target = match.group(1), match.group(2)
    metadata = current_app.hg.get(target)
    if metadata is None:
        if title is not None:
            target = title
        return [paragraph(rawtext, target)], []
    url = url_for('views.content', slug=metadata.slug, _external=True)
    if title is None:
        title = metadata.title
    return [reference(url, title, refuri=url)], []


index = {
    'attachment': attachment_role,
    'page': page_role,
}



# -*- coding: utf-8 -*-
"""
    blohg.script
    ~~~~~~~~~~~~

    Module with the CLI script related stuff.

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

import os
import sys
import posixpath
from flask.ext.script import Command, Manager, Server, Option
from flask_frozen import Freezer, MissingURLGeneratorWarning
from warnings import filterwarnings
from werkzeug.routing import Map

from blohg import create_app
from blohg.utils import create_repo

# filter MissingURLGeneratorWarning warnings.
filterwarnings('ignore', category=MissingURLGeneratorWarning)


class InitRepo(Command):
    """initialize a blohg repo, using the default template."""

    def handle(self, app):
        try:
            create_repo(app)
        except RuntimeError, err:
            print >> sys.stderr, str(err)


class Freeze(Command):
    """ freeze the blog into a set of static files. """

    option_list = (Option('--serve', '-s', dest='serve', default=False,
                          action='store_true'),
                   Option('--noindex', dest='no_index', default=False,
                          action='store_true'))

    def remap_rules(self, map, map_html):
        """remaping the rules with files extensions"""
        mapping = {'views.source': 'txt',
                   'views.atom': 'atom'}
        if map_html:
            mapping['views.tag'] = 'html'
            mapping['views.content'] = 'html'
            mapping['views.post_list'] = 'html'
            mapping['views.posts'] = 'html'
            mapping['views.home'] = 'html'
        rules = []
        for rule in map.iter_rules():
            rule = rule.empty()
            if rule.is_leaf:
                # Add the leafs without modif.
                rules.append(rule)
                continue

            # special treatment for the robot.txt url
            if rule.rule == '/source/':
                rules.append(rule)
                continue

            try:
                extension = mapping[rule.endpoint]
            except KeyError:
                # the rest can go through
                rules.append(rule)
                continue
            # It becomes a leaf
            rule.is_leaf = True
            # and we add an extension
            url = rule.rule[:-1]
            if url == '':
                url = '/index'
            rule.rule = url + '.' + extension
            # and we add the modified rule
            rules.append(rule)
        return Map(rules)

    def handle(self, app, serve, no_index):

        app.url_map = self.remap_rules(app.url_map, no_index)

        # That's a risky one, it woud be better to give a parameter to the
        # freezer
        app.root_path = app.config.get('REPO_PATH')

        freezer = Freezer(app)

        def static_generator(static_dir):
            for f in app.hg.revision.manifest():
                if f.startswith(static_dir):
                    yield dict(filename=f[len(static_dir):] \
                               .strip(posixpath.sep))

        @freezer.register_generator
        def static():
            """Walk the static dir and freeze everything"""
            return static_generator(app.config['STATIC_DIR'])

        @freezer.register_generator
        def attachments():
            """Walk the attachment dir and freeze everything"""
            return static_generator(app.config['ATTACHMENT_DIR'])

        freezer.freeze()
        if serve:
            freezer.serve()


def create_script():
    """Script object factory

    :param repo_path: the path to the mercurial repository.
    :return: the script object (Flask-Script' Manager instance).
    """

    script = Manager(create_app, with_default_commands=False)
    script.add_option('-r', '--repo-path', dest='repo_path',
                      default=os.getcwd(), required=False)
    server = Server(use_debugger=True, use_reloader=True)
    server.description = 'runs the blohg local server.'
    script.add_command('runserver', server)
    script.add_command('initrepo', InitRepo())
    script.add_command('freeze', Freeze())

    return script



# -*- coding: utf-8 -*-
"""
    blohg.hgapi.templates
    ~~~~~~~~~~~~~~~~~~~~~

    Module with stuff to deal with Jinja2 templates from Mercurial
    repositories.

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

import os
import posixpath

from flask import current_app
from jinja2.loaders import BaseLoader, TemplateNotFound, split_template_path


class MercurialLoader(BaseLoader):
    """A Jinja2 loader that loads templates from a Mercurial repository"""

    def _filerev(self, filectx):
        filelog = filectx.filelog()
        revision_list = list(filelog)
        if len(revision_list) == 0:
            revision_id = 0
        else:
            revision_id = revision_list[-1]
        return filelog.linkrev(revision_id)

    def get_source(self, environment, template):
        pieces = split_template_path(template)
        templates_dir = current_app.config['TEMPLATES_DIR']
        filename = posixpath.join(templates_dir, *pieces)
        if filename in current_app.hg.revision.manifest():
            filectx = current_app.hg.revision[filename]
            contents = filectx.data().decode('utf-8')
            revision = self._filerev(filectx)

            def up2date():
                if current_app.hg.revision.rev() is None or revision == -1:
                    return False
                return revision >= \
                       self._filerev(current_app.hg.revision[filename])

            return contents, os.path.join(templates_dir, *pieces), up2date
        raise TemplateNotFound(template)

    def list_templates(self):
        templates_dir = current_app.config['TEMPLATES_DIR'].strip(os.linesep)
        return sorted([i[len(templates_dir) + 1:] for i in \
                       current_app.hg.revision.manifest() \
                       if i.startswith(templates_dir + '/')])



# -*- coding: utf-8 -*-
"""
    blohg.tests.hgapi
    ~~~~~~~~~~~~~~~~~

    Module with tests for blohg integration with mercurial.

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

import codecs
import os
import posixpath
from jinja2.loaders import ChoiceLoader
from mercurial import commands

from blohg.tests.utils import TestCaseWithNewRepo


class HgApiTestCase(TestCaseWithNewRepo):

    def test_repo_structure(self):
        for f in [posixpath.join('content', 'attachments', 'mercurial.png'),
                  posixpath.join('content', 'post', 'example-post.rst'),
                  posixpath.join('content', 'post', 'lorem-ipsum.rst'),
                  posixpath.join('content', 'about.rst'),
                  posixpath.join('static', 'screen.css'),
                  posixpath.join('templates', 'base.html'),
                  posixpath.join('templates', 'posts.html'),
                  posixpath.join('templates', 'post_list.html'),
                  'config.yaml', '.hgignore']:
            self.assertTrue(f in self.app.hg.revision.manifest(),
                            'File not found: %s' % f)

    def test_setup_mercurial(self):
        self.assertTrue(hasattr(self.app, 'hg'), 'mercurial not setup.')
        self.assertTrue(isinstance(self.app.jinja_loader, ChoiceLoader),
                        'Invalid Jinja2 loader.')

    def test_get_page(self):
        with self.app.test_request_context():
            data = self.app.hg.get('about')
            self.assertFalse(data is None, 'Page not found: about')
            self.assertTrue('About me' in data.title,
                            'Failed to parse the title')
            self.assertTrue('<img' in data.full_html,
                            'reStructuredText parser failed')
            self.assertEqual(data.path, 'content/about.rst', 'Invalid path')
            self.assertEqual(data.slug, 'about', 'Failed to parse the slug')
            self.assertEqual(data.author_name, 'foo',
                             'Failed to parse author name')
            self.assertEqual(data.author_email, 'foo@bar.com',
                             'Failed to parse author email')
            self.assertEqual(data.description, 'This is an example page!')
            self.assertTrue('mercurial.png' in data.images[0])

            # TODO: test aliases

    def test_get_post(self):
        with self.app.test_request_context():
            data = self.app.hg.get('post/lorem-ipsum')
            self.assertFalse(data is None, 'Page not found: post/lorem-ipsum')
            self.assertTrue('lorem-ipsum' in data.tags,
                            'Failed to parse the tags')

    def test_reload(self):
        with self.app.test_request_context():
            data = self.app.hg.get('post/foo-bar')
            self.assertTrue(data is None,
                            'New post found before created! ' \
                            'Run to the hills :P')

        new_file = os.path.join(self.repo_path, 'content', 'post',
                                'foo-bar.rst')
        with codecs.open(new_file, 'w', encoding='utf-8') as fp:
            fp.write(os.linesep.join(['Foo', '---', '', 'Bar']))
        commands.add(self.ui, self.repo)

        self.app.hg.reload()
        with self.app.test_request_context():
            data = self.app.hg.get('post/foo-bar')
            self.assertTrue(data is None, 'Reload failed. Uncommited stuff ' \
                            'was reloaded, with debug disabled')

        self.app.debug = True
        self.app.hg.reload()
        with self.app.test_request_context():
            data = self.app.hg.get('post/foo-bar')
            self.assertFalse(data is None, 'Reload failed. Uncommited stuff ' \
                             'wasn\'t reloaded, with debug enabled')

        self.app.debug = False
        self.commit(message='foo', addremove=True)
        self.app.hg.reload()
        with self.app.test_request_context():
            data = self.app.hg.get('post/foo-bar')
            self.assertFalse(data is None, 'Reload failed. Commited stuff ' \
                             'wasn\'t reloaded')



# -*- coding: utf-8 -*-
"""
    blohg.rst.writer
    ~~~~~~~~~~~~~~~~

    Module with the custom blohg reStructuredText HTML writer.

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

from docutils.writers.html4css1 import HTMLTranslator, Writer

import re


class BlohgWriter(Writer):

    def __init__(self):
        Writer.__init__(self)
        self.translator_class = BlohgHTMLTranslator

    def assemble_parts(self):
        # we will add 2 new parts to the writer: 'first_paragraph_as_text' and
        # 'images'
        Writer.assemble_parts(self)
        self.parts['first_paragraph_as_text'] = \
            self.visitor.first_paragraph_as_text
        self.parts['images'] = self.visitor.images


class BlohgHTMLTranslator(HTMLTranslator):

    def __init__(self, document):
        HTMLTranslator.__init__(self, document)
        self.first_paragraph_as_text = None
        self.images = []

    def visit_iframe_flash_video(self, node):
        if 'thumbnail_uri' in node:
            self.images.append(node['thumbnail_uri'])
        atts = dict(src=node['uri'])
        if 'width' in node:
            atts['width'] = node['width']
        if 'height' in node:
            atts['height'] = node['height']
        if 'border' in node:
            atts['frameborder'] = node['border']
        if 'allowfullscreen' in node and node['allowfullscreen'] == 'true':
            # weird behavior of <iframe>, I'm not sure if this thing actually
            # works at all
            atts['allowfullscreen'] = 'true'
        style = []
        for att_name in 'width', 'height':
            if att_name in atts:
                if re.match(r'^[0-9.]+$', atts[att_name]):
                    # Interpret unitless values as pixels.
                    atts[att_name] += 'px'
                style.append('%s: %s;' % (att_name, atts[att_name]))
                del atts[att_name]
        if style:
            atts['style'] = ' '.join(style)
        if 'align' in node:
            atts['class'] = 'align-%s' % node['align']
        self.body.append(self.starttag(node, 'iframe', '', **atts) + \
                         '</iframe>\n')

    def depart_iframe_flash_video(self, node):
        pass

    def visit_paragraph(self, node):
        if self.first_paragraph_as_text is None:
            # just paragraphs at the root of the document should be used.
            # warnings, blockquotes and similar aren't good descriptions.
            if node.parent.tagname.strip() == 'document':
                self.first_paragraph_as_text = \
                    ' '.join([i.strip() \
                              for i in node.astext().splitlines(False)])
        HTMLTranslator.visit_paragraph(self, node)

    def visit_image(self, node):
        # opengraph specs allows multiple images.
        # http://ogp.me/#array
        self.images.append(node['uri'])
        HTMLTranslator.visit_image(self, node)



# -*- coding: utf-8 -*-
"""
    blohg.views
    ~~~~~~~~~~~

    A generic Flask Module with all the views.

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

import math

from flask import Blueprint, abort, current_app, make_response, \
     render_template, url_for, redirect
from jinja2 import TemplateNotFound
from werkzeug.contrib.atom import AtomFeed, FeedEntry
from werkzeug.exceptions import NotFound

views = Blueprint('views', __name__)


# this is like a stack. I want 'atom' to be used by url_for, then it should be
# in the latest decorators
@views.route('/rss/')  # dumb redirect to keep the compatibility
@views.route('/rss/<path:tag>/')  # dumb redirect to keep the compatibility
@views.route('/atom/')
@views.route('/atom/<path:tag>/')
def atom(tag=None):
    """General purpose atom feed."""

    title = current_app.config['TITLE']
    if tag is not None:
        tags = tag.split('/')
        for _tag in tags:
            if _tag not in current_app.hg.tags:
                abort(404)
        title += u' » %s' % ' + '.join(tags)
        posts = current_app.hg.get_by_tag(tags)
    else:
        posts = current_app.hg.get_all(True)
    feed = AtomFeed(title=title, subtitle=current_app.config['TAGLINE'],
                    url=url_for('views.home', _external=True),
                    id=url_for('views.atom', tag=tag),
                    feed_url=url_for('views.atom', tag=tag, _external=True),
                    author=current_app.config['AUTHOR'],
                    generator=('blohg', None, None))
    for post in posts[:int(current_app.config['POSTS_PER_PAGE'])]:
        feed.add(FeedEntry(title=post.title, content=post.full_raw_html,
                           summary=post.abstract_raw_html,
                           id=url_for('views.content', slug=post.slug),
                           url=url_for('views.content', slug=post.slug,
                                       _external=True),
                           author=dict(name=post.author_name,
                                       email=post.author_email),
                           published=post.datetime, updated=post.datetime))
    return feed


@views.route('/<path:slug>/')
def content(slug):
    """Posts and static pages."""
    page = current_app.hg.get(slug)
    if page is None:
        url = "/%s/" % slug
        if url in current_app.hg.aliases:
            code, slug = current_app.hg.aliases[url]
            return redirect(url_for('views.content', slug=slug), code=code)
        abort(404)
    title = page.title
    if slug.startswith('post'):
        title = u'Post: %s' % page.title
    return render_template('_posts.html', title=title, posts=[page],
                           full_content=True, meta=page)


@views.route('/')
@views.route('/page/<int:page>/')
def home(page=None):
    """Page with the abstract of the posts. Part of the pagination."""
    if page is None:
        try:
            return content('index')
        except NotFound:
            page = 1
    current = int(page)
    pages = current_app.hg.get_all(True)
    ppp = int(current_app.config['POSTS_PER_PAGE'])
    num_pages = int(math.ceil(float(len(pages)) / ppp))
    init = int((current - 1) * ppp)
    end = int(current * ppp)
    return render_template('_posts.html', posts=pages[init:end],
                           full_content=False,
                           pagination={'num_pages': num_pages,
                                       'current': page})


@views.route('/posts/')
def posts():
    """An alias to ``/page/1/``. Not adding it as a decorator to the existing
    view to provide another endpoint.
    """
    return home(1)


@views.route('/post/')
def post_list():
    """Page with a simple list of posts (link + creation date)."""
    try:
        return render_template('post_list.html', title=u'Posts',
                               posts=current_app.hg.get_all(True))
    except TemplateNotFound:
        if 'FREEZER_BASE_URL' in current_app.config:  # freezing the blog
            return ""
        abort(404)


@views.route('/tag/<path:tag>/')
def tag(tag):
    """Page that lists the abstract of all available posts for the given
    tag.
    """
    tags = tag.split('/')
    for _tag in tags:
        if _tag not in current_app.hg.tags:
            abort(404)
    posts = current_app.hg.get_by_tag(tags)
    return render_template('_posts.html', title=u'Tag: %s' % ' + '.join(tags),
                           tag=tags, posts=posts, full_content=False)


@views.route('/source/')  # just to make robots.txt's url_for happy :)
@views.route('/source/<path:slug>/')
def source(slug=None):
    """View that shows the source code of a given static page/post. Can be
    disabled setting SHOW_RST_SOURCE configuration parameter to ``False``
    """
    if 'FREEZER_BASE_URL' in current_app.config and slug is None:  # freezing
        return ''
    if not current_app.config['SHOW_RST_SOURCE'] or slug is None:
        abort(404)
    source = current_app.hg.get(slug)
    if source is None:
        abort(404)
    response = make_response(source.full)
    response.headers['Content-Type'] = 'text/plain; charset=utf-8'
    return response


@views.route('/robots.txt')
def robots():
    """View that generates a robots.txt file for ``/source/``  path, to avoid
    source files to be indexed by search engines. Can be disabled setting
    ROBOTS_TXT configuration parameter to ``False``
    """
    if not current_app.config['ROBOTS_TXT']:
        abort(404)
    response = make_response(render_template('robots.txt'))
    response.headers['Content-Type'] = 'text/plain; charset=utf-8'
    return response



# -*- coding: utf-8 -*-
"""
    blohg.rst.nodes
    ~~~~~~~~~~~~~~~

    Module with the custom blohg reStructuredText doctree nodes.

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

from docutils.nodes import General, Element


class iframe_flash_video(General, Element):
    pass



# -*- coding: utf-8 -*-
"""
    blohg.rst
    ~~~~~~~~~

    Package with reStructuredText-related stuff needed by blohg, like
    directives and roles.

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

from docutils.core import publish_parts
from docutils.parsers.rst.directives import register_directive
from docutils.parsers.rst.roles import register_local_role
from flask import current_app

from blohg.rst.directives import index as directives_index
from blohg.rst.roles import index as roles_index
from blohg.rst.writer import BlohgWriter

# registering docutils' directives
for directive in directives_index:
    register_directive(directive, directives_index[directive])

# registering docutils' roles
for role in roles_index:
    register_local_role(role, roles_index[role])


def parser(content):
    settings = {'input_encoding': 'utf-8', 'output_encoding': 'utf-8',
                'initial_header_level': current_app.config['RST_HEADER_LEVEL']}
    parts = publish_parts(source=content, writer=BlohgWriter(),
                          settings_overrides=settings)
    return {'title': parts['title'], 'fragment': parts['fragment'],
            'first_paragraph_as_text': parts['first_paragraph_as_text'],
            'images': parts['images']}



# -*- coding: utf-8 -*-
"""
    blohg.rst.directives
    ~~~~~~~~~~~~~~~~~~~~

    Module with the custom blohg reStructuredText directives.

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

from docutils import nodes, statemachine
from docutils.utils.error_reporting import ErrorString
from docutils.io import FileInput
from docutils.parsers.rst import directives, Directive
from docutils.parsers.rst.directives.images import Image, Figure
from docutils.parsers.rst.directives.misc import Include
from flask import current_app, request, url_for
from pygments import highlight
from pygments.formatters import HtmlFormatter
from pygments.lexers import get_lexer_by_name, TextLexer
from urllib import pathname2url

from blohg.hgapi.io import MercurialFile
from blohg.rst.nodes import iframe_flash_video

import posixpath


__all__ = ['Vimeo', 'Youtube', 'Math', 'Code', 'SourceCode', 'AttachmentImage',
           'AttachmentFigure', 'SubPages', 'IncludeHg']

GOOGLETEX_URL = 'https://chart.googleapis.com/chart?cht=tx&chl='


def align(argument):
    return directives.choice(argument, ('left', 'center', 'right'))


def boolean(argument):
    return directives.choice(argument, ('true', 'false'))


class Vimeo(Directive):
    """reStructuredText directive that creates an embed object to display
    a video from Vimeo

    Usage example::

        .. vimeo:: QFwQIRwuAM0
           :align: center
           :height: 344
           :width: 425
    """

    required_arguments = 1
    optional_arguments = 0
    option_spec = {
        'height': directives.length_or_unitless,
        'width': directives.length_or_percentage_or_unitless,
        'border': directives.length_or_unitless,
        'align': align,
        'allowfullscreen': boolean,
    }
    has_content = False

    def run(self):
        self.options['uri'] = 'http://player.vimeo.com/video/' \
            + self.arguments[0]
        self.options.setdefault('width', '425px')
        self.options.setdefault('height', '344px')
        self.options.setdefault('align', 'center')
        self.options.setdefault('border', '0')
        self.options.setdefault('allowfullscreen', 'true')
        return [iframe_flash_video('', **self.options)]


class Youtube(Directive):
    """reStructuredText directive that creates an embed object to display
    a video from Youtube

    Usage example::

        .. youtube:: QFwQIRwuAM0
           :align: center
           :height: 344
           :width: 425
    """

    required_arguments = 1
    optional_arguments = 0
    option_spec = {
        'height': directives.length_or_unitless,
        'width': directives.length_or_percentage_or_unitless,
        'border': directives.length_or_unitless,
        'align': align,
        'allowfullscreen': boolean,
    }
    has_content = False

    def run(self):
        self.options['uri'] = 'http://www.youtube.com/embed/%s' % \
            self.arguments[0]
        self.options['thumbnail_uri'] = \
            'http://img.youtube.com/vi/%s/hqdefault.jpg' % self.arguments[0]
        self.options.setdefault('width', '425px')
        self.options.setdefault('height', '344px')
        self.options.setdefault('align', 'center')
        self.options.setdefault('border', '0')
        self.options.setdefault('allowfullscreen', 'true')
        return [iframe_flash_video('', **self.options)]


class Code(Directive):
    """reStructuredText directive that creates a pre tag suitable for
    decoration with http://alexgorbatchev.com/SyntaxHighlighter/

    Usage example::

        .. source:: python

           print "Hello, World!"

        .. raw:: html

            <script type="text/javascript" src="http://alexgorbatchev.com/pub/sh/current/scripts/shCore.js"></script>
            <script type="text/javascript" src="http://alexgorbatchev.com/pub/sh/current/scripts/shBrushPython.js"></script>
            <link type="text/css" rel="stylesheet" href="http://alexgorbatchev.com/pub/sh/current/styles/shCoreDefault.css"/>
            <script type="text/javascript">SyntaxHighlighter.defaults.toolbar=false; SyntaxHighlighter.all();</script>

    """

    required_arguments = 1
    optional_arguments = 0
    has_content = True

    def run(self):
        self.assert_has_content()
        self.options['brush'] = self.arguments[0]
        html = '''\

<pre class="brush: %s">
%s
</pre>

'''
        return [nodes.raw('', html % (self.options['brush'],
            "\n".join(self.content).replace('<', '&lt;')),
            format='html')]


class SourceCode(Directive):
    """reStructuredText directive that does syntax highlight using Pygments.

    Usage example::

        .. sourcecode:: python
           :linenos:

            print "Hello, World!"

    The ``linenos`` option enables the line numbering.

    To be able to use this directive you should generate a CSS file with the
    style definitions, using the ``pygmentize`` script, shipped with Pygments.

    ::

        $ pygmentyze -S friendly -f html > static/pygments.css

    Where ``friendly`` will be your Pygments style of choice.

    This file should be included in the main template, usually ``base.html``::

        <link type="text/css" media="screen" rel="stylesheet" href="{{
            url_for('static', filename='pygments.css') }}" />

    This directive is based on ``rst-directive.py``, created by Pygments
    authors.
    """

    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {
        'linenos': directives.flag,
    }
    has_content = True

    def run(self):
        self.assert_has_content()
        try:
            lexer = get_lexer_by_name(self.arguments[0])
        except ValueError:
            # no lexer found - use the text one instead of an exception
            lexer = TextLexer()
        formatter = HtmlFormatter(noclasses=False)
        if 'linenos' in self.options:
            formatter.linenos = 2  # inline
        parsed = highlight(u'\n'.join(self.content), lexer, formatter)
        return [nodes.raw('', parsed, format='html')]


class Math(Image):
    """reStructuredText directive that creates an image HTML object to
    display a LaTeX equation, using Google Chart API.

    Usage example::

        .. math::

            \frac{x^2}{1+x}
    """

    required_arguments = 0
    has_content = True

    def run(self):
        self.assert_has_content()
        if not 'align' in self.options:
            self.options['align'] = 'center'
        tmp = pathname2url(' '.join([(i == '' and '\\\\' or i.strip()) \
            for i in self.content]))
        self.arguments.append('%s%s' % (GOOGLETEX_URL, tmp))
        return Image.run(self)


class AttachmentImage(Image):

    def run(self):
        my_file = directives.uri(self.arguments[0])
        full_path = posixpath.join(current_app.config['ATTACHMENT_DIR'],
                                   my_file)
        if full_path not in current_app.hg.revision.manifest():
            raise self.error(
                'Error in "%s" directive: File not found: %s.' % (
                    self.name, full_path
                )
            )
        self.arguments[0] = url_for('attachments', filename=my_file,
                                    _external=True)
        return Image.run(self)


class AttachmentFigure(Figure):

    def run(self):
        my_file = directives.uri(self.arguments[0])
        full_path = posixpath.join(current_app.config['ATTACHMENT_DIR'],
                                   my_file)
        if full_path not in current_app.hg.revision.manifest():
            raise self.error(
                'Error in "%s" directive: File not found: %s.' % (
                    self.name, full_path
                )
            )
        self.arguments[0] = url_for('attachments', filename=my_file,
                                    _external=True)
        return Figure.run(self)


class SubPages(Directive):
    """reStructuredText directive that creates a bullet-list with the subpages
    of the current page, or of a given page.

    Usage example::

        .. subpages::

    Or::

        .. subpages:: projects

    Supposing that you have a directory called ``content/projects`` and some
    reStructuredText files on it. Subdirectories are also allowed.

    It is also possible to change the way the bullet-list is sorted, using the
    options ``sort-by`` and ``sort-order``::

        .. subpages::
           :sort-by: slug
           :sort-order: desc

    Available options for ``sort-by`` are ``slug`` (default option), ``title``
    and ``date``, and for ``sort-order`` are ``asc`` (default option) and
    ``desc``.

    This directive will just show the files from the root of the directory.
    It's not recursive.
    """

    required_arguments = 0
    optional_arguments = 1
    option_spec = {
        'sort-by': lambda x: directives.choice(x, ('slug', 'title', 'date')),
        'sort-order': lambda x: directives.choice(x, ('asc', 'desc'))
    }
    has_content = False

    def run(self):
        self.options.setdefault('sort-by', 'slug')
        self.options.setdefault('sort-order', 'asc')
        if len(self.arguments) == 0:
            self.arguments.append(request.path.strip('/'))
        tmp_metadata = []
        final_metadata = []
        # not sure if this works on windows
        splited_dir = self.arguments[0].split('/')
        for metadata in current_app.hg.get_all():
            # not sure if this works on windows
            splited_slug = metadata.slug.split('/')
            if metadata.slug.startswith(self.arguments[0]) and \
               (len(splited_dir) + 1 == len(splited_slug)):
                tmp_metadata.append(metadata)
        for metadata in sorted(tmp_metadata, key=lambda x: \
                               getattr(x, self.options['sort-by']),
                               reverse=(self.options['sort-order'] == 'desc')):
            link = url_for('views.content', slug=metadata.slug)
            reference = nodes.reference(link, metadata.title, refuri=link)
            final_metadata.append(nodes.list_item('',
                                                  nodes.paragraph('', '',
                                                                  reference)))
        return [nodes.bullet_list('', *final_metadata)]


class IncludeHg(Include):

    def run(self):
        if not self.state.document.settings.file_insertion_enabled:
            raise self.warning('"%s" directive disabled.' % self.name)
        path = directives.path(self.arguments[0])
        # ALL the included files are relative to the repository root.
        # we need to remove absolute paths
        if path.startswith('/'):
            raise self.severe('Problem with "%s" directive path:\npath ' \
                              'should be relative' % self.name)
        encoding = self.options.get(
            'encoding', self.state.document.settings.input_encoding)
        tab_width = self.options.get(
            'tab-width', self.state.document.settings.tab_width)
        try:
            self.state.document.settings.record_dependencies.add(path)
            include_file = FileInput(
                source=MercurialFile(path), encoding=encoding,
                error_handler=(self.state.document.settings.\
                               input_encoding_error_handler),
                handle_io_errors=None)
        except IOError, error:
            raise self.severe(u'Problems with "%s" directive path:\n%s.' %
                      (self.name, ErrorString(error)))
        startline = self.options.get('start-line', None)
        endline = self.options.get('end-line', None)
        try:
            if startline or (endline is not None):
                lines = include_file.readlines()
                rawtext = ''.join(lines[startline:endline])
            else:
                rawtext = include_file.read()
        except UnicodeError, error:
            raise self.severe(u'Problem with "%s" directive:\n%s' %
                              (self.name, ErrorString(error)))
        # start-after/end-before: no restrictions on newlines in match-text,
        # and no restrictions on matching inside lines vs. line boundaries
        after_text = self.options.get('start-after', None)
        if after_text:
            # skip content in rawtext before *and incl.* a matching text
            after_index = rawtext.find(after_text)
            if after_index < 0:
                raise self.severe('Problem with "start-after" option of "%s" '
                                  'directive:\nText not found.' % self.name)
            rawtext = rawtext[after_index + len(after_text):]
        before_text = self.options.get('end-before', None)
        if before_text:
            # skip content in rawtext after *and incl.* a matching text
            before_index = rawtext.find(before_text)
            if before_index < 0:
                raise self.severe('Problem with "end-before" option of "%s" '
                                  'directive:\nText not found.' % self.name)
            rawtext = rawtext[:before_index]
        if 'literal' in self.options:
            # Convert tabs to spaces, if `tab_width` is positive.
            if tab_width >= 0:
                text = rawtext.expandtabs(tab_width)
            else:
                text = rawtext
            literal_block = nodes.literal_block(rawtext, text, source=path)
            literal_block.line = 1
            return [literal_block]
        else:
            include_lines = statemachine.string2lines(
                rawtext, tab_width, convert_whitespace=1)
            self.state_machine.insert_input(include_lines, path)
            return []


index = {
    'vimeo': Vimeo,
    'youtube': Youtube,
    'math': Math,
    'code': Code,
    'sourcecode': SourceCode,
    'attachment-image': AttachmentImage,
    'attachment-figure': AttachmentFigure,
    'subpages': SubPages,
    'include-hg': IncludeHg,
    'include': IncludeHg,
}



# -*- coding: utf-8 -*-
"""
    blohg.hgapi.static
    ~~~~~~~~~~~~~~~~~~

    Module with stuff to deal with static files from Mercurial repositories.

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

from flask import current_app, request, abort
from time import time
from zlib import adler32

import posixpath
import mimetypes


class MercurialStaticFile(object):
    """Callable to create a Response object for static files loaded from
    the current Mercurial repository.
    """

    def __init__(self, config_key):
        self._config_key = config_key

    def __call__(self, filename):
        directory = current_app.config[self._config_key]
        filename = posixpath.join(directory, filename)
        mimetype = mimetypes.guess_type(filename)[0]
        if mimetype is None:
            mimetype = 'application/octet-stream'
        try:
            filectx = current_app.hg.revision[filename]
            data = filectx.data()
        except:
            abort(404)
        rv = current_app.response_class(data, mimetype=mimetype,
            direct_passthrough=True)
        rv.cache_control.public = True
        cache_timeout = 60 * 60 * 12
        rv.cache_control.max_age = cache_timeout
        rv.expires = int(time() + cache_timeout)
        try:
            date = int(filectx.date()[0])
        except:
            date = time()
        rv.set_etag('blohg-%s-%s-%s' % (date, len(data), adler32(filename) \
            & 0xffffffff))
        return rv.make_conditional(request)



# -*- coding: utf-8 -*-
"""
    blohg.version
    ~~~~~~~~~~~~~

    Module with the version number related stuff.

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

import re


# CHANGE ME BEFORE THE RELEASE!
# when this file got installed from a mercurial repository, the hash of the
# current changeset is appended to the 'version' variable.
version = '0.11.2'

# the file isn't installed but this isn't a stable release, then we may be
# running from a mercurial repository. let's verify.
if re.match(r'^[0-9.]+\+$', version):

    # we don't need to care about use mercurial from command-line, because we
    # are GPL already anyway :)
    from mercurial import hg, ui as hgui
    from mercurial.commands import identify
    import os

    # root dir is where the parent dir of the dir where this file is.
    root_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), '..')

    # if we are inside a mercurial repository...
    if os.path.isdir(os.path.join(root_dir, '.hg')):

        # create an ui object
        ui = hgui.ui()

        # create the repo object
        repo = hg.repository(ui, root_dir)

        # use the ui stack to get the return value from the 'hg identify'
        # command, and run the command itself.
        ui.pushbuffer()
        identify(ui, repo, id=True)
        rv = ui.popbuffer()

        # append the changeset hash to the version.
        if rv:
            version += '/' + rv.strip()



# -*- coding: utf-8 -*-
"""
    blohg.tests
    ~~~~~~~~~~~

    Main unit test package.

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

import unittest
from blohg.tests.hgapi import HgApiTestCase
from blohg.tests.hgapi.templates import MercurialLoaderTestCase


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(HgApiTestCase))
    suite.addTest(unittest.makeSuite(MercurialLoaderTestCase))
    return suite



# -*- coding: utf-8 -*-
"""
    blohg.hgapi.models
    ~~~~~~~~~~~~~~~~~~

    Module with the basic classes for the mercurial content handling.

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

import re
import time

from datetime import datetime
from flask.helpers import locked_cached_property
from jinja2 import Markup
from mercurial import encoding

from blohg.rst import parser
from blohg.utils import parse_date

re_metadata = re.compile(r'\.\. +([a-z]+): *(.+)')
re_read_more = re.compile(r'\.\. +read_more')
re_author = re.compile(r'^(?P<name>[^<]*[^ ])( ?<(?P<email>[^<]*)>)?$')


def hg2u(s):
    """ returns a unicode object representing the mercurial string """
    return encoding.fromlocal(s).decode("utf-8")


class Page(object):
    """Pages are the very basic content element of a blog. They don't have tags
    nor other fancy stuff that belongs to posts."""

    def __init__(self, parent, filectx):
        self._parent = parent
        self._filectx = filectx
        self._filecontent = filectx.data()
        self._vars = {}
        self._title = None

        # get metadata variables from rst source
        for i in re_metadata.finditer(self._filecontent):
            self._vars[i.group(1)] = hg2u(i.group(2).strip())

        # handle aliases
        if 'aliases' in self._vars:
            self._vars['aliases'] = [i.strip() for i in \
                                     self._vars['aliases'].split(',')]

        # get the creation date, from the time of the first changeset with this
        # content or from the 'date' variable.
        filelog = self._filectx.filelog()
        changesets = list(filelog)
        first_changeset = self._parent.repo[filelog.linkrev(0)]
        if 'date' in self._vars:
            self._vars['date'] = parse_date(self._vars['date'])
        else:
            self._vars['date'] = int(first_changeset.date()[0])
            if self._vars['date'] == 0:
                self._vars['date'] = int(time.time())
        self._vars['datetime'] = datetime.utcfromtimestamp(self._vars['date'])

        # get the modification date, from the time of the last changeset with
        # changes for this content, from the creation date, if this content was
        # never changed, or from the 'mdate' variable.
        if 'mdate' in self._vars:
            self._vars['mdate'] = parse_date(self._vars['mdate'])
        if 'mdate' not in self._vars and len(changesets) > 1:
            last_changeset = self._parent.repo[filelog.linkrev(
                len(changesets) - 1)]
            self._vars['mdate'] = int(last_changeset.date()[0])
        if 'mdate' in self._vars:
            self._vars['mdatetime'] = \
                datetime.utcfromtimestamp(self._vars['mdate'])

        # get the author name/email from the 'author' variable or from the
        # commiter of this content to the repository.
        if 'author' not in self._vars:
            try:
                self._vars['author'] = hg2u(first_changeset.user())
            except:
                del self._vars['author']
            else:
                if self._vars['author'] == u'':
                    try:
                        self._vars['author'] = hg2u(self._filectx.user())
                    except:
                        del self._vars['author']

    @locked_cached_property
    def parsed_source(self):
        return parser(self.full)

    @locked_cached_property
    def parsed_abstract(self):
        if not self.read_more:
            return self.parsed_source
        return parser(self.abstract)

    @locked_cached_property
    def parsed_author(self):
        if 'author' not in self._vars:
            return {}
        rv = re_author.match(self._vars['author'])
        if rv is None:
            return {'name': self._vars['author']}
        return rv.groupdict()

    @locked_cached_property
    def title(self):
        return self._vars.get('title', self.parsed_source['title'])

    @locked_cached_property
    def description(self):
        return self._vars.get('description',
                              self.parsed_source['first_paragraph_as_text'])

    @locked_cached_property
    def images(self):
        return self.parsed_source.get('images', [])

    @locked_cached_property
    def flash_videos(self):
        return self.parsed_source.get('flash_videos', [])

    @locked_cached_property
    def author_name(self):
        if 'name' in self.parsed_author:
            return self.parsed_author['name']

    @locked_cached_property
    def author_email(self):
        if 'email' in self.parsed_author:
            return self.parsed_author['email']

    @locked_cached_property
    def path(self):
        return self._filectx.path()

    @locked_cached_property
    def slug(self):
        rv = re.match(r'^' + self._parent.content_dir + r'[\\/](.+)' +
                      '\\' + self._parent.post_ext + '$',
                      self._filectx.path())
        if rv is not None:
            return rv.group(1)

    @locked_cached_property
    def aliases(self):
        return self._vars.get('aliases', [])

    @locked_cached_property
    def abstract(self):
        return hg2u(re_read_more.split(self._filecontent)[0])

    @locked_cached_property
    def abstract_html(self):
        return Markup(self.parsed_abstract['fragment'])

    @locked_cached_property
    def abstract_raw_html(self):
        return self.parsed_abstract['fragment']

    @locked_cached_property
    def full(self):
        return hg2u(self._filecontent)

    @locked_cached_property
    def full_html(self):
        return Markup(self.parsed_source['fragment'])

    @locked_cached_property
    def full_raw_html(self):
        return self.parsed_source['fragment']

    @locked_cached_property
    def read_more(self):
        return len(re_read_more.split(self._filecontent)) > 1

    def get(self, key, default=None):
        return self._vars.get(key, default)

    def __getattr__(self, attr):
        return self.get(attr)

    def __str__(self):
        return self.full

    def __repr__(self):
        return '<%s %r>' % (self.__class__.__name__, self.slug)


class Post(Page):
    """Posts are like pages, but with tags support."""

    def __init__(self, parent, filectx):
        Page.__init__(self, parent, filectx)

        # handle tags
        if 'tags' in self._vars:
            self._vars['tags'] = [i.strip() for i in \
                                  self._vars['tags'].split(',')]

    @locked_cached_property
    def tags(self):
        return self._vars.get('tags', [])



# -*- coding: utf-8 -*-
"""
    blohg.hgapi.io
    ~~~~~~~~~~~~~~

    File-like objects for Mercurial.

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

from flask import current_app

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO


class MercurialFile(object):
    """A file-like read-only object for mercurial file-context objects.

    Based in {c,}StringIO. Poorly implemented, because we can't inherit classes
    implemented in C libraries, like cStringIO.StringIO.
    """

    def __init__(self, path):
        try:
            self._fctx = current_app.hg.revision[path]
        except:
            # IncludeHg catches IOError
            raise IOError('Failed to read file from repository: %s' % path)
        self._obj = StringIO(self._fctx.data())

    @property
    def name(self):
        return self._fctx.path()

    def __getattr__(self, name):
        return getattr(self._obj, name)



# -*- coding: utf-8 -*-
"""
    blohg.tests.utils
    ~~~~~~~~~~~~~~~~~

    Module with utilities for blohg tests.

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

import os
import unittest
from mercurial import hg, ui, commands
from shutil import rmtree
from tempfile import mkdtemp

from blohg import create_app
from blohg.utils import create_repo


def walk(p):
    def _walk(entries, path):
        for entry in os.listdir(path):
            full_entry = os.path.join(path, entry)
            if os.path.isdir(full_entry):
                _walk(entries, full_entry)
            else:
                entries.append(full_entry)
    e = []
    _walk(e, p)
    return e


class TestCaseWithNewRepo(unittest.TestCase):

    def setUp(self):
        self.repo_path = mkdtemp()
        self.ui = ui.ui()
        self.ui.setconfig('ui', 'quiet', True)
        self.ui.setconfig('ui', 'username', 'foo <foo@bar.com>')
        self.app = create_app(self.repo_path, hgui=self.ui)
        create_repo(self.app)
        self.repo = hg.repository(self.ui, self.repo_path)
        commands.add(self.ui, self.repo)
        self.app.hg.reload()

    def tearDown(self):
        try:
            rmtree(self.repo_path)
        except:
            pass

    def add(self, *pats, **opts):
        return commands.add(self.ui, self.repo, *pats, **opts)

    def commit(self, *pats, **opts):
        return commands.commit(self.ui, self.repo, *pats, **opts)



# -*- coding: utf-8 -*-
"""
    blohg.tests.hgapi.templates
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Module with tests for blohg integration with jinja2.

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

import codecs
import os
from jinja2.exceptions import TemplateNotFound
from mercurial import commands

from blohg.tests.utils import TestCaseWithNewRepo, walk


class MercurialLoaderTestCase(TestCaseWithNewRepo):

    def test_get_source(self):
        with self.app.test_request_context():
            self.assertRaises(TemplateNotFound,
                              self.app.jinja_loader.get_source,
                              self.app.jinja_env, 'test.html')

        new_file = os.path.join(self.repo_path,
                                self.app.config['TEMPLATES_DIR'], 'test.html')

        with codecs.open(new_file, 'w', encoding='utf-8') as fp:
            fp.write('foo')
        commands.add(self.ui, self.repo)

        self.app.hg.reload()
        with self.app.test_request_context():
            self.assertRaises(TemplateNotFound,
                              self.app.jinja_loader.get_source,
                              self.app.jinja_env, 'test.html')

        self.app.debug = True
        self.app.hg.reload()
        with self.app.test_request_context():
            contents, filename, up2date = self.app.jinja_loader.get_source(
                self.app.jinja_env, 'test.html')
            self.assertTrue('foo' in contents, 'Invalid template!')
            # TODO: fix and test filename
            self.assertFalse(up2date(), 'up2date failed. It should always ' \
                             'return False when debug is enabled')

        self.app.debug = False
        self.app.hg.reload()
        with self.app.test_request_context():
            contents, filename, up2date = self.app.jinja_loader.get_source(
                self.app.jinja_env, 'test.html')
            self.assertFalse(up2date(), 'up2date failed. Debug disabled. ' \
                             'Before commit.')
            self.commit(message='foo', addremove=True)
            self.app.hg.reload()
            contents, filename, up2date = self.app.jinja_loader.get_source(
                self.app.jinja_env, 'test.html')
            self.assertTrue(up2date(), 'up2date failed. Debug disabled. ' \
                            'After commit.')
            with codecs.open(new_file, 'a', encoding='utf-8') as fp:
                fp.write('bar')
            self.commit(message='bar', addremove=True)
            self.app.hg.reload()
            self.assertFalse(up2date(), 'up2date failed. Debug disabled. ' \
                             'After 2nd commit')

    def test_list_templates(self):
        default_templates_dir = os.path.join(os.path.dirname(
            os.path.abspath(__file__)), '..', '..', 'templates')
        templates_dir = os.path.join(self.repo_path,
                                     self.app.config['TEMPLATES_DIR'])
        e = walk(default_templates_dir) + walk(templates_dir)
        with self.app.test_request_context():
            for f in self.app.jinja_loader.list_templates():
                full_f1 = os.path.join(default_templates_dir, f)
                full_f2 = os.path.join(templates_dir, f)
                self.assertTrue(full_f1 in e or full_f2 in e,
                                'File not found: %s' % f)



# -*- coding: utf-8 -*-
"""
    blohg.hgapi
    ~~~~~~~~~~~

    Package with Mercurial-related stuff needed by blohg to load posts
    and static pages.

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

import re
import time
import yaml

from jinja2.loaders import ChoiceLoader
from mercurial import hg, ui, cmdutil
from blohg.hgapi.models import Page, Post
from blohg.hgapi.static import MercurialStaticFile
from blohg.hgapi.templates import MercurialLoader


def setup_mercurial(app, hgui=None):
    """This function adds a :class:`Hg` instance to an application object, as a
    ``hg`` attribute, and reloads it as needed.

    :param app: the application object, must have a 'REPO_PATH' configuration
                parameter.
    :param hgui: a Mercurial ui object.
    """

    # create an ui object and attach the Hg object to app
    app.hgui = hgui or ui.ui()
    app.hg = Hg(app)

    # setup our jinja2 custom loader and static file handlers
    old_loader = app.jinja_loader
    app.jinja_loader = ChoiceLoader([MercurialLoader(), old_loader])
    app.add_url_rule(app.static_url_path + '/<path:filename>',
                     endpoint='static',
                     view_func=MercurialStaticFile('STATIC_DIR'))
    app.add_url_rule('/attachments/<path:filename>', endpoint='attachments',
                     view_func=MercurialStaticFile('ATTACHMENT_DIR'))

    @app.before_request
    def before_request():
        app.hg.reload()


class Hg(object):
    """Main class of the Mercurial layer."""

    def __init__(self, app, config_file=None):
        self.app = app
        self.config_file = config_file or 'config.yaml'
        self.repo = None
        self.revision = None
        self.revision_id = None
        self.default_branch = None
        self.content_dir = None
        self.post_ext = None
        self.pages = []
        self.posts = []
        self.tags = set()
        self.aliases = {}
        self.reload()

    def _parse_aliases(self, post_or_page):
        for alias in post_or_page.aliases:
            code = 302
            alias = alias.encode('utf-8')
            if alias[:4] in ('301:', '302:'):
                code = int(alias[:3])
                alias = alias[4:]
            self.aliases[alias.encode('utf-8')] = (code, post_or_page.slug)

    def reload(self):
        """Method to reload stuff from the Mercurial repository. It is able to
        reload as needed, to save resources.
        """
        needed = False

        # a new repo object is always needed, for reloads or vefify the if the
        # current stuff is new enough
        repopath = self.app.config.get('REPO_PATH', '.')
        repopath = cmdutil.findrepo(repopath)

        # if findrepo returns None we don't have a repo (yet).
        if repopath is None:
            return

        repo = hg.repository(self.app.hgui, repopath)
        try:
            default_branch = repo.branchtags()['default']
        except KeyError:
            default_branch = None

        # initialize it if needed
        if self.repo is None or self.revision is None:
            needed = True

        # debug mode wants a reload
        if self.app.debug:
            needed = True

        # if we don't need a refresh yet, let's verify if our current revision
        # is new enough
        if not needed:
            if repo[default_branch].rev() != \
               self.repo[self.default_branch].rev():
                needed = True

        if not needed:
            return

        # check which stuff we want.
        #     None: uncommited stuff.
        #     default: last commited stuff.
        revision_id = None
        if not self.app.debug:
            revision_id = default_branch

        # register our shiny new stuff
        self.repo = repo
        self.revision = repo[revision_id]
        self.revision_id = revision_id
        self.default_branch = default_branch

        # load configs
        if self.config_file not in self.revision.manifest():
            raise RuntimeError('Configuration file not found: %s' % \
                               self.config_file)
        config = self.revision[self.config_file].data()
        self.app.config.update(yaml.load(config))

        # monkey-patch configs for debug
        if self.app.debug:
            if 'GOOGLE_ANALYTICS' in self.app.config:
                del self.app.config['GOOGLE_ANALYTICS']
            self.app.config['DISQUS_DEVELOPER'] = True

        # build a regular expression for search posts/pages.
        self.content_dir = self.app.config.get('CONTENT_DIR', 'content')
        self.post_ext = self.app.config.get('POST_EXT', '.rst')
        re_content = re.compile(r'^' + self.content_dir + \
                                r'[\\/](post)?.+' + '\\' + self.post_ext + '$')

        # get all the content
        self.pages = []
        self.posts = []
        self.tags = set()
        self.aliases = {}
        now = int(time.time())
        for fname in self.revision.manifest():
            rv = re_content.match(fname)
            if rv is not None:
                if rv.group(1) is None:  # page
                    page = Page(self, self.revision[fname])
                    if not self.app.debug and page.date > now:
                        continue
                    self._parse_aliases(page)
                    self.pages.append(page)
                else:  # post
                    post = Post(self, self.revision[fname])
                    if not self.app.debug and post.date > now:
                        continue
                    self._parse_aliases(post)
                    self.posts.append(post)
                    self.tags = self.tags.union(set(post.tags))

        # sort tags by "name" and convert them back to a list
        self.tags = sorted(self.tags)

        # sort posts reverse by date. sort pages is useless :P
        self.posts = sorted(self.posts, lambda a, b: b.date - a.date)

    def get(self, slug):
        """Method that returns a :class:`Page` or a :class:`Post` object for
        a given slug.

        :param slug: the slug string.
        :return: a :class:`Page` or a :class:`Post`
        """
        for entry in self.posts + self.pages:
            if entry.slug == slug:
                return entry

    def get_all(self, only_posts=False):
        """Method that returns a list of :class:`Page` or :class:`Post`
        objects for all the available files.

        :param only_posts: a boolean that makes the method returns only
                           the available posts for the given locale, not
                           the static pages.
        :return: a list of :class:`Page` or :class:`Post` objects.
        """
        if only_posts:
            return self.posts
        return self.posts + self.pages

    def get_by_tag(self, tag):
        """Method that returns a list of :class:`Post` objects for a
        given list of tag identifier strings.

        :param tag: a list of tag identifier strings.
        :return: a list of :class:`Post` objects.
        """
        posts = []
        for post in self.posts:
            found = True
            for _tag in tag:
                if _tag not in post.tags:
                    found = False
            if found:
                posts.append(post)
        return posts

    def __repr__(self):
        return '<%s %r>' % (self.__class__.__name__, self.repo.root)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    setup.py
    ~~~~~~~~

    :copyright: (c) 2010-2012 by Rafael Goncalves Martins
    :license: GPL-2, see LICENSE for more details.
"""

import codecs
import re
from setuptools import setup, find_packages
from setuptools.command.install_lib import install_lib as _install_lib

# doing things the wrong way...
# we need the module blohg.version but we can't import the full package
# first time because the dependencies probably aren't solved yet.
import os
import sys
cwd = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.join(cwd, 'blohg'))
from version import version as __version__


# based on code from flask's make-release.py script:
# https://github.com/mitsuhiko/flask/blob/master/scripts/make-release.py
class install_lib(_install_lib):
    """Custom ``install_lib`` phase to patch blohg.version in order to make
    the value calculated on that module permanent for installations. This
    value is usually the hash of a Mercurial changeset, when installing blohg
    from a Mercurial clone."""

    # let's hook our patch at the end of the install phase.
    def install(self):

        # run default install phase
        outfiles = _install_lib.install(self)

        # does nothing if we aren't installing from a live version
        if not re.match(r'^[0-9.]+\+/', __version__):
            return outfiles

        # calculate file path
        path = os.path.join(self.install_dir, 'blohg', 'version.py')

        changed = []

        # helper callback to build the new version string
        def build_version(match):
            changed.append(True)
            return match.group(1) + __version__ + match.group(3)

        # open the file and patch
        with codecs.open(path, encoding='utf-8') as fp:
            contents = re.sub(r"(\s*version\s*=\s*')(.+?)(')(?sm)",
                              build_version, fp.read())

        # if the file was sucessfully patched, save it again
        if changed:
            with codecs.open(path, 'w', encoding='utf-8') as fp:
                fp.write(contents)
        else:
            self.warn('Failed to set version.')

        return outfiles


setup(
    name='blohg',
    version=__version__,
    license='GPL-2',
    description='A Mercurial-based blogging engine',
    long_description=open(os.path.join(cwd, 'README.rst')).read(),
    author='Rafael Goncalves Martins',
    author_email='rafael@rafaelmartins.eng.br',
    url='http://blohg.org/',
    platforms='any',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'docutils==0.10',
        'Flask>=0.8',
        'Flask-Babel>=0.7',
        'Flask-Script>=0.3',
        'Frozen-Flask>=0.7',
        'Jinja2>=2.5.2',
        'Mercurial>=1.6',
        'PyYAML',
        'Pygments',  # TODO: make this dependency optional
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content :: News/Diary',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
        'Topic :: Software Development :: Version Control',
    ],
    data_files=[
        ('share/blohg', ['share/blohg.wsgi']),
    ],
    scripts=['bin/blohg'],
    cmdclass={'install_lib': install_lib},
    test_suite='blohg.tests.suite',
)
